/*
 * Process execution defines.
 *
 * Copyright (C) 1998-2003 Hewlett-Packard Co
 *	David Mosberger-Tang <davidm@hpl.hp.com>
 * Copyright (C) 1999 Asit Mallick <asit.k.mallick@intel.com>
 * Copyright (C) 1999 Don Dugger <don.dugger@intel.com>
 */
#ifndef _ASM_IA64_EXEC_H
#define _ASM_IA64_EXEC_H

#define arch_align_stack(x) (x)

#endif /* _ASM_IA64_EXEC_H */
