#ifndef __SOUND_STAC946X_H
#define __SOUND_STAC946X_H

#define STAC946X_RESET			0x00
#define STAC946X_STATUS			0x01
#define STAC946X_MASTER_VOLUME		0x02
#define STAC946X_LF_VOLUME		0x03
#define STAC946X_RF_VOLUME		0x04
#define STAC946X_LR_VOLUME		0x05
#define STAC946X_RR_VOLUME		0x06
#define STAC946X_CENTER_VOLUME		0x07
#define STAC946X_LFE_VOLUME		0x08
#define STAC946X_MIC_L_VOLUME		0x09
#define STAC946X_MIC_R_VOLUME		0x0a
#define STAC946X_DEEMPHASIS		0x0c
#define STAC946X_GENERAL_PURPOSE	0x0d
#define STAC946X_AUDIO_PORT_CONTROL	0x0e
#define STAC946X_MASTER_CLOCKING	0x0f
#define STAC946X_POWERDOWN_CTRL1	0x10
#define STAC946X_POWERDOWN_CTRL2	0x11
#define STAC946X_REVISION_CODE		0x12
#define STAC946X_ADDRESS_CONTROL	0x13
#define STAC946X_ADDRESS		0x14

#endif  /*  __SOUND_STAC946X_H */
