/*
 * Copyright (C) 2008 Analog Devices Inc.
 * Licensed under the GPL-2 or later.
 */


#ifndef _MACH_GPIO_H_
#define _MACH_GPIO_H_

#define MAX_BLACKFIN_GPIOS 48

#define GPIO_PF0	0
#define GPIO_PF1	1
#define GPIO_PF2	2
#define GPIO_PF3	3
#define GPIO_PF4	4
#define GPIO_PF5	5
#define GPIO_PF6	6
#define GPIO_PF7	7
#define GPIO_PF8	8
#define GPIO_PF9	9
#define GPIO_PF10	10
#define GPIO_PF11	11
#define GPIO_PF12	12
#define GPIO_PF13	13
#define GPIO_PF14	14
#define GPIO_PF15	15
#define GPIO_PF16	16
#define GPIO_PF17	17
#define GPIO_PF18	18
#define GPIO_PF19	19
#define GPIO_PF20	20
#define GPIO_PF21	21
#define GPIO_PF22	22
#define GPIO_PF23	23
#define GPIO_PF24	24
#define GPIO_PF25	25
#define GPIO_PF26	26
#define GPIO_PF27	27
#define GPIO_PF28	28
#define GPIO_PF29	29
#define GPIO_PF30	30
#define GPIO_PF31	31
#define GPIO_PF32	32
#define GPIO_PF33	33
#define GPIO_PF34	34
#define GPIO_PF35	35
#define GPIO_PF36	36
#define GPIO_PF37	37
#define GPIO_PF38	38
#define GPIO_PF39	39
#define GPIO_PF40	40
#define GPIO_PF41	41
#define GPIO_PF42	42
#define GPIO_PF43	43
#define GPIO_PF44	44
#define GPIO_PF45	45
#define GPIO_PF46	46
#define GPIO_PF47	47

#define PORT_FIO0 GPIO_PF0
#define PORT_FIO1 GPIO_PF16
#define PORT_FIO2 GPIO_PF32

#include <mach-common/ports-f.h>

#endif /* _MACH_GPIO_H_ */
