/*
 * include/asm-sh/cpu-sh2/freq.h
 *
 * Copyright (C) 2006  Yoshinori Sato
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 */
#ifndef __ASM_CPU_SH2_FREQ_H
#define __ASM_CPU_SH2_FREQ_H

#if defined(CONFIG_CPU_SUBTYPE_SH7619)
#define FREQCR	0xf815ff80
#endif

#endif /* __ASM_CPU_SH2_FREQ_H */

