/*
 * include/asm-xtensa/hw_irq.h
 *
 * This file is subject to the terms and conditions of the GNU General
 * Public License.  See the file "COPYING" in the main directory of
 * this archive for more details.
 *
 * Copyright (C) 2002 - 2005 Tensilica Inc.
 */

#ifndef _XTENSA_HW_IRQ_H
#define _XTENSA_HW_IRQ_H

#endif
