/*
 * Port J Masks
 */

#ifndef __BFIN_PERIPHERAL_PORT_J__
#define __BFIN_PERIPHERAL_PORT_J__

#define PJ0		(1 << 0)
#define PJ1		(1 << 1)
#define PJ2		(1 << 2)
#define PJ3		(1 << 3)
#define PJ4		(1 << 4)
#define PJ5		(1 << 5)
#define PJ6		(1 << 6)
#define PJ7		(1 << 7)
#define PJ8		(1 << 8)
#define PJ9		(1 << 9)
#define PJ10		(1 << 10)
#define PJ11		(1 << 11)
#define PJ12		(1 << 12)
#define PJ13		(1 << 13)
#define PJ14		(1 << 14)
#define PJ15		(1 << 15)

#endif
