#ifndef _ASM_IA64_MC146818RTC_H
#define _ASM_IA64_MC146818RTC_H

/*
 * Machine dependent access functions for RTC registers.
 */

/* empty include file to satisfy the include in genrtc.c */

#endif /* _ASM_IA64_MC146818RTC_H */
