/*
 * arch/arm/mach-shark/include/mach/irqs.h
 *
 * by Alexander Schulz
 */

#define NR_IRQS			16

#define IRQ_ISA_KEYBOARD	 1
#define IRQ_ISA_RTC_ALARM	 8
#define I8042_KBD_IRQ		 1
#define I8042_AUX_IRQ		12
#define IRQ_HARDDISK            14
