/*
 * arch/arm/plat-omap/include/plat/l3_3xxx.h - L3 firewall definitions
 *
 * Copyright (C) 2010 Texas Instruments Incorporated - http://www.ti.com/
 *	Sumit Semwal
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 */
#ifndef __ARCH_ARM_PLAT_OMAP_INCLUDE_PLAT_L3_3XXX_H
#define __ARCH_ARM_PLAT_OMAP_INCLUDE_PLAT_L3_3XXX_H

/* L3 Initiator IDs */
/* Display Sub system (DSS) */
#define OMAP3_L3_CORE_FW_INIT_ID_DSS			29

#endif
