/*
 * include/asm-xtensa/bugs.h
 *
 * This is included by init/main.c to check for architecture-dependent bugs.
 *
 * Xtensa processors don't have any bugs.  :)
 *
 * This file is subject to the terms and conditions of the GNU General
 * Public License.  See the file "COPYING" in the main directory of
 * this archive for more details.
 */

#ifndef _XTENSA_BUGS_H
#define _XTENSA_BUGS_H

static void check_bugs(void) { }

#endif /* _XTENSA_BUGS_H */
