/*
 * Copyright (C) 2008 Analog Devices Inc.
 * Licensed under the GPL-2 or later.
 */


#ifndef _MACH_GPIO_H_
#define _MACH_GPIO_H_

#define MAX_BLACKFIN_GPIOS 16

#define GPIO_PF0	0
#define GPIO_PF1	1
#define GPIO_PF2	2
#define GPIO_PF3	3
#define GPIO_PF4	4
#define GPIO_PF5	5
#define GPIO_PF6	6
#define GPIO_PF7	7
#define GPIO_PF8	8
#define GPIO_PF9	9
#define GPIO_PF10	10
#define GPIO_PF11	11
#define GPIO_PF12	12
#define GPIO_PF13	13
#define GPIO_PF14	14
#define GPIO_PF15	15

#define PORT_F GPIO_PF0

#include <mach-common/ports-f.h>

#endif /* _MACH_GPIO_H_ */
