/*
 * arch/arm/mach-iop32x/include/mach/iq80321.h
 *
 * Intel IQ80321 evaluation board registers
 */

#ifndef __IQ80321_H
#define __IQ80321_H

#define IQ80321_UART		0xfe800000	/* UART #1 */
#define IQ80321_7SEG_1		0xfe840000	/* 7-Segment MSB */
#define IQ80321_7SEG_0		0xfe850000	/* 7-Segment LSB (WO) */
#define IQ80321_ROTARY_SW	0xfe8d0000	/* Rotary Switch */
#define IQ80321_BATT_STAT	0xfe8f0000	/* Battery Status */


#endif
