#ifndef __XTENSA_S6105_HARDWARE_H
#define __XTENSA_S6105_HARDWARE_H

#define PLATFORM_DEFAULT_MEM_START	0x40000000
#define PLATFORM_DEFAULT_MEM_SIZE	0x08000000

#define MAX_DMA_ADDRESS			0

#define KERNELOFFSET			(PLATFORM_DEFAULT_MEM_START + 0x1000)

#endif /* __XTENSA_S6105_HARDWARE_H */
